from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class SubnetPublicIP(BaseResourceNegativeValueCheck):
    def get_forbidden_values(self):
        return [True]

    def get_inspected_key(self):
        return "map_public_ip_on_launch"

    def __init__(self):
        name = "Ensure VPC subnets do not assign public IP by default"
        id = "CKV_AWS_130"
        supported_resources = ['aws_subnet']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)


check = SubnetPublicIP()



from enum import Enum


class SolverType(str, Enum):
    ATTRIBUTE = "ATTRIBUTE"
    COMPLEX = "COMPLEX"
    CONNECTION = "CONNECTION"
    COMPLEX_CONNECTION = "COMPLEX_CONNECTION"
    FILTER = "FILTER"



from .contains_attribute_solver import ContainsAttributeSolver


class NotContainsAttributeSolver(ContainsAttributeSolver):
    operator = 'not_contains'

    def __init__(self, resource_types, attribute, value):
        super().__init__(resource_types=resource_types,
                         attribute=attribute, value=value)

    def _get_operation(self, vertex, attribute):
        return not super(NotContainsAttributeSolver, self)._get_operation(vertex, attribute)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ELBv2AccessLogs(BaseResourceCheck):
    def __init__(self):
        name = "Ensure the ELBv2 (Application/Network) has access logging enabled"
        id = "CKV_AWS_91"
        supported_resources = ['AWS::ElasticLoadBalancingV2::LoadBalancer']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'Properties' in conf.keys():
            if 'LoadBalancerAttributes' in conf['Properties'].keys():
                if isinstance(conf['Properties']['LoadBalancerAttributes'], list):
                    for item in conf['Properties']['LoadBalancerAttributes']:
                        if 'Key' in item.keys() and 'Value' in item.keys():
                            if item['Key'] == "access_logs.s3.enabled" and item['Value'] == "true":
                                return CheckResult.PASSED
        return CheckResult.FAILED


check = ELBv2AccessLogs()



from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class LBDeletionProtection(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that Load Balancer has deletion protection enabled"
        id = "CKV_AWS_113"
        supported_resources = ['aws_lb', 'aws_alb']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'enable_deletion_protection'


check = LBDeletionProtection()



from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class KubernetesSecretsEncryptedUsingCMK(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Kubernetes Secrets are encrypted using Customer Master Keys (CMKs) managed in AWS KMS"
        id = "CKV_AWS_106"
        supported_resources = ['aws_eks_cluster']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'encryption_config' not in conf.keys():
            return CheckResult.FAILED
        else:
            encryption_config = conf.get('encryption_config')[0]
            if 'resources' in encryption_config and 'provider' in encryption_config:
                resources = encryption_config.get('resources')[0]
                provider = encryption_config.get('provider')[0]
                if type(resources) is list:
                    if 'secrets' in resources and 'key_arn' in provider:
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = KubernetesSecretsEncryptedUsingCMK()



from checkov.terraform.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver
from functools import reduce
from operator import or_


class OrSolver(BaseComplexSolver):
    operator = 'or'

    def __init__(self, solvers, resource_types):
        super().__init__(solvers, resource_types)

    def _get_operation(self, *args):
        return reduce(or_, args)

    def get_operation(self, vertex):
        for i, solver in enumerate(self.solvers):
            pred_result = solver.get_operation(vertex)
            if pred_result:
                return pred_result
        return False



from networkx.classes.digraph import DiGraph
from checkov.common.graph.checks_infra.enums import SolverType
from checkov.terraform.checks_infra.solvers.connections_solvers.complex_connection_solver import ComplexConnectionSolver
from checkov.terraform.graph_builder.graph_components.attribute_names import CustomAttributes


class OrConnectionSolver(ComplexConnectionSolver):
    operator = 'or'

    def __init__(self, solvers, operator):
        super().__init__(solvers, operator)

    def get_operation(self, graph_connector: DiGraph):
        passed_attributes, failed_attributes = self.run_attribute_solvers(graph_connector)
        failed_attributes = [f for f in failed_attributes if
                             f[CustomAttributes.ID] not in [p[CustomAttributes.ID] for p in passed_attributes]]
        passed, failed = passed_attributes, failed_attributes
        connection_solvers = [sub_solver for sub_solver in self.solvers if
                              sub_solver.solver_type in [SolverType.CONNECTION, SolverType.COMPLEX_CONNECTION]]
        passed_connections, failed_connections = [], []
        failed_by_hash = {}
        for connection_solver in connection_solvers:
            connection_solver.set_vertices(graph_connector, [])
            passed_solver, failed_solvers = connection_solver.get_operation(graph_connector)
            passed_connections.extend(passed_solver)
            for f in failed_solvers:
                if f[CustomAttributes.ID] not in [p[CustomAttributes.ID] for p in passed_connections]:
                    if not failed_by_hash.get(f[CustomAttributes.HASH]):
                        failed_by_hash[f[CustomAttributes.HASH]] = {"v": f, "count": 0}
                    failed_by_hash[f[CustomAttributes.HASH]]["count"] += 1

        for data in failed_by_hash.values():
            if data["count"] == len(connection_solvers) or data["v"] not in passed_connections:
                failed.append(data["v"])

        passed.extend(passed_connections)
        failed = [f for f in failed if
                  f[CustomAttributes.ID] not in [p[CustomAttributes.ID] for p in passed]]

        return self.filter_results(passed, failed)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.base_spec_check import BaseK8Check


class ReadinessProbe(BaseK8Check):

    def __init__(self):
        name = "Readiness Probe Should be Configured"
        id = "CKV_K8S_9"
        # initContainers do not need Readiness Probes...
        # Location: container .readinessProbe
        supported_kind = ['containers']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def get_resource_id(self, conf):
        return f'{conf["parent"]} - {conf["name"]}'

    def scan_spec_conf(self, conf):
        # Don't check Job/CronJob (or Pods in runtime derived from Job/CronJob)
        if "parent" in conf:
            if "Job" in conf["parent"]:
                return CheckResult.PASSED
            if "parent_metadata" in conf:
                if "ownerReferences" in conf["parent_metadata"]:
                    for ref in conf["parent_metadata"]["ownerReferences"]:
                        if ref["kind"] == "Job":
                            return CheckResult.PASSED
        if "readinessProbe" not in conf:
            return CheckResult.FAILED
        return CheckResult.PASSED


check = ReadinessProbe()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import json


class S3AllowsAnyPrincipal(BaseResourceCheck):

    def __init__(self):
        name = "Ensure S3 bucket does not allow an action with any Principal"
        id = "CKV_AWS_70"
        supported_resources = ['aws_s3_bucket', 'aws_s3_bucket_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'policy' in conf.keys():
            if isinstance(conf['policy'][0], str):
                try:
                    policy_block = json.loads(conf['policy'][0])
                    if 'Statement' in policy_block.keys():
                        for statement in force_list(policy_block['Statement']):
                            if statement['Effect'] == 'Deny':
                                continue
                            if 'Principal' not in statement:
                                continue

                            principal = statement['Principal']

                            if principal == '*':
                                return CheckResult.FAILED
                            elif 'AWS' in statement['Principal']:
                                # Can be a string or an array of strings
                                aws = statement['Principal']['AWS']
                                if (type(aws) == str and aws == '*') or (type(aws) == list and '*' in aws):
                                    return CheckResult.FAILED
                except: # nosec
                    pass
        return CheckResult.PASSED


check = S3AllowsAnyPrincipal()



from checkov.terraform.context_parsers.base_parser import BaseContextParser
import dpath.util


class LocalsContextParser(BaseContextParser):
    def __init__(self):
        definition_type = "locals"
        super().__init__(definition_type=definition_type)

    def _collect_local_values(self, local_block):
        if isinstance(local_block,dict):
            for local_name, local_value in local_block.items():
                local_value = local_value[0] if isinstance(local_value, list) and len(local_value) > 0 else local_value
                if type(local_value) in (int, float, bool, str, dict):
                    dpath.new(self.context, ['assignments', local_name], local_value)

    def get_block_type(self):
        return self.definition_type

    def get_entity_context_path(self, entity_block):
        return []

    def enrich_definition_block(self, definition_blocks):
        self.context = super().enrich_definition_block(definition_blocks)
        for i, locals_block in enumerate(definition_blocks):
            self._collect_local_values(locals_block)
        return self.context


parser = LocalsContextParser()



import logging
import os
from typing import List

from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.graph_components.blocks import Block


def convert_graph_vertices_to_tf_definitions(vertices: List[Block], root_folder):
    tf_definitions = {}
    breadcrumbs = {}
    for vertex in vertices:
        block_path = vertex.path
        if not os.path.isfile(block_path):
            print(f'tried to convert vertex to tf_definitions but its path doesnt exist: {vertex}')
            continue
        tf_path = block_path
        if vertex.module_dependency:
            tf_path = f"{block_path}[{vertex.module_dependency}#{vertex.module_dependency_num}]"
        block_type = vertex.block_type.value
        if block_type == BlockType.TF_VARIABLE:
            continue
        if tf_definitions.get(tf_path) is None:
            tf_definitions[tf_path] = {}
        if tf_definitions.get(tf_path).get(block_type) is None:
            tf_definitions.get(tf_path)[block_type] = []
        tf_definitions.get(tf_path)[block_type].append(vertex.config)
        relative_block_path = f"/{os.path.relpath(block_path if not vertex.module_dependency else vertex.module_dependency, root_folder)}"
        add_breadcrumbs(vertex, breadcrumbs, relative_block_path)
    logging.info(f'breadcrumbs = {breadcrumbs}')
    return tf_definitions, breadcrumbs


def add_breadcrumbs(vertex: Block, breadcrumbs, relative_block_path):
    vertex_breadcrumbs = vertex.breadcrumbs
    if vertex_breadcrumbs:
        if breadcrumbs.get(relative_block_path) is None:
            breadcrumbs[relative_block_path] = {}
        breadcrumbs[relative_block_path][vertex.name] = vertex_breadcrumbs



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GoogleCloudPostgreSqlLogMinMessage(BaseResourceCheck):
    def __init__(self):
        name = "Ensure PostgreSQL database 'log_min_messages' flag is set to a valid value"
        check_id = "CKV_GCP_55"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for google_sql_database_instance which has valid value in log_min_messages flag on PostgreSQL DBs::
            :param
            conf: google_sql_database_instance
            configuration
            :return: < CheckResult >
        """
        if 'database_version' in conf.keys():
            key = conf['database_version'][0]
            if 'POSTGRES' in key:
                if 'settings' in conf.keys():
                    for attribute in conf['settings'][0]:
                        if attribute == 'database_flags':
                            flags = conf['settings'][0]['database_flags']
                            if isinstance(flags[0], list): #treating use cases of the following database_flags parsing (list of list of dictionaries with strings):'database_flags': [[{'name': '<key>', 'value': '<value>'}, {'name': '<key>', 'value': '<value>'}]]
                                flags = conf['settings'][0]['database_flags'][0]
                                for flag in flags:
                                    if (flag['name'] == 'log_min_messages'):
                                        key_logmin = flag['value']
                                        logmin_list = ['fatal', 'panic', 'log', 'error', 'warning', 'notice',
                                                       'info', 'debug1', 'debug2', 'debug3', 'debug4', 'debug5']
                                        if key_logmin not in logmin_list:
                                            return CheckResult.FAILED
                            else: #treating use cases of the following database_flags parsing (list of dictionaries with arrays): 'database_flags': [{'name': ['<key>'], 'value': ['<value>']},{'name': ['<key>'], 'value': ['<value>']}]
                                for flag in flags:
                                    if (flag['name'][0] == 'log_min_messages'):
                                        key_logmin = flag['value'][0]
                                        logmin_list = ['fatal', 'panic', 'log', 'error', 'warning', 'notice',
                                                       'info', 'debug1', 'debug2', 'debug3', 'debug4', 'debug5']
                                        if key_logmin not in logmin_list:
                                            return CheckResult.FAILED
        return CheckResult.PASSED


check = GoogleCloudPostgreSqlLogMinMessage()
